/* signal.h
 *
 *  signal.h,v 1.3 1995/12/19 20:00:35 joel Exp
 */

#ifndef __POSIX_SIGNALS_h
#define __POSIX_SIGNALS_h

#include <rtems/posix/features.h>

#include <sys/time.h>
#include <sys/siginfo.h>

/*
 *  3.3.2 Send a Signal to a Process, P1003.1b-1993, p. 68
 *
 *  NOTE: Behavior of kill() depends on _POSIX_SAVED_IDS.
 */

int kill(
  int   pid_t,
  int   sig
);

/*
 *  3.3.3 Manipulate Signal Sets, P1003.1b-1993, p. 69
 */

int sigemptyset(
  sigset_t   *set
);

int sigfillset(
  sigset_t   *set
);

int sigaddset(
  sigset_t   *set,
  int         signo
);

int sigdelset(
  sigset_t   *set,
  int         signo
);

int sigismembet(
  const sigset_t   *set,
  int               signo
);

/*
 *  3.3.4 Examine and Change Signal Action, P1003.1b-1993, p. 70
 */

int sigaction(
  int                     sig,
  const struct sigaction *act,
  struct sigaction       *oact
);

/*
 *  3.3.5 Examine and Change Blocked Signals, P1003.1b-1993, p. 73
 *
 *  NOTE: P1003.1c/D10, p. 37 adds pthread_sigmask().
 */

/* values for how */

#define SIG_BLOCK   0 /* resulting_set = (current_set | set)  */
#define SIG_UNBLOCK 0 /* resulting_set = (current_set & ~set)  */ 
#define SIG_SETMASK 0 /* resulting_set = set */

int sigprocmask(
  int               how,
  const sigset_t   *set,
  sigset_t         *oset
);

#if defined(_POSIX_THREADS)

int pthread_sigmask(
  int               how,
  const sigset_t   *set,
  sigset_t         *oset
);

#endif

/*
 *  3.3.6 Examine Pending Signals, P1003.1b-1993, p. 75
 */

int sigpending(
  sigset_t  *set
);

/*
 *  3.3.7 Wait for a Signal, P1003.1b-1993, p. 75
 */

int sigsuspend(
  const sigset_t  *sigmask
);

#if defined(_POSIX_REALTIME_SIGNALS)

/*
 *  3.3.8 Synchronously Accept a Signal, P1003.1b-1993, p. 76
 *
 *  NOTE: P1003.1c/D10, p. 39 adds sigwait().
 */

int sigwaitinfo(
  const sigset_t  *set,
  siginfo_t       *info
);

int sigtimedwait(
  const sigset_t         *set,
  siginfo_t              *info,
  const struct timespec  *timeout
);

int sigwait(
  const sigset_t  *set,
  int             *sig
);

/*
 *  3.3.9 Queue a Signal to a Process, P1003.1b-1993, p. 78
 */

int sigqueue(
  pid_t               pid,
  int                 signo,
  const union sigval  value
);

#endif

/*
 *  3.3.10 Send a Signal to a Thread, P1003.1c/D10, p. 43
 */

#if defined(_POSIX_THREADS)

int pthread_kill(
  pthread_t   thread,
  int         sig
);

#endif

/*
 *  3.4.1 Schedule Alarm, P1003.1b-1993, p. 79
 */

unsigned int alarm(
  unsigned int seconds
);

/*
 *  3.4.2 Suspend Process Execution, P1003.1b-1993, p. 80
 */

int pause( void );

/*
 *  3.4.3 Delay Process Execution, P1003.1b-1993, p. 73
 */

unsigned int sleep(
  unsigned int seconds
);

#endif
/* end of include file */
