/* time.h
 *
 *  time.h,v 1.3 1995/12/19 20:00:37 joel Exp
 */

#ifndef __POSIX_TIME_h
#define __POSIX_TIME_h

#include <rtems/posix/features.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/siginfo.h>

#if defined(_POSIX_THREADS)
#include <sys/pthread.h>
#endif

/*
 *  4.8.1.5 Special Symbol {CLK_TCK}, P1003.1b-1993, p. 97
 */

#define CLK_TCK  sysconf(_SC_CLK_TCK)

/*
 *  4.5.1 Get System Time, P1003.1b-1993, p. 91
 */

time_t time(
  time_t   *tloc
);

#if defined(_POSIX_TIMERS)

/*
 *  14.2.1 Clocks, P1003.1b-1993, p. 263
 */

int clock_settime(
  clockid_t              clock_id,
  const struct timespec *tp
);

int clock_gettime(
  clockid_t        clock_id,
  struct timespec *tp
);

int clock_getres(
  clockid_t        clock_id,
  struct timespec *res
);

/*
 *  14.2.2 Create a Per-Process Timer, P1003.1b-1993, p. 264
 */

int timer_create(
  clockid_t        clock_id,
  struct sigevent *evp,
  timer_t         *timerid
);

/*
 *  14.2.3 Delete a Per_process Timer, P1003.1b-1993, p. 266
 */

int timer_delete(
  timer_t timerid
);

/*
 *  14.2.4 Per-Process Timers, P1003.1b-1993, p. 267
 */

int timer_settime(
  timer_t                  timerid,
  int                      flags,
  const struct itimerspec *value,
  struct itimerspec       *ovalue
);

int timer_gettime(
  timer_t            timerid,
  struct itimerspec *value
);

int timer_getoverrun(
  timer_t   timerid
);

/*
 *  14.2.5 High Resolution Sleep, P1003.1b-1993, p. 269
 */

int nanosleep(
  const struct timespec  *rqtp,
  struct timespec        *rmtp
);

#endif /* _POSIX_TIMERS */

/*
 *  20.1.1 CPU-time Clock Attributes, P1003.4b/D8, p. 54
 */

/* values for the clock enable atribute */

#define CLOCK_ENABLED  0  /* clock is enabled, i.e. counting execution time */
#define CLOCK_DISABLED 0  /* clock is disabled */

/* values for the pthread cputime_clock_allowed attribute */

#define CLOCK_ALLOWED    0 /* If a thread is created with this value a */
                           /*   CPU-time clock attached to that thread */
                           /*   shall be accessible. */
#define CLOCK_DISALLOWED 0 /* If a thread is created with this value, the */
                           /*   thread shall not have a CPU-time clock */
                           /*   accessible. */

/*
 *  20.1.2 Manifest Constants, P1003.4b/D8, p. 55
 */

#if defined(_POSIX_CPUTIME)
 
/*
 *  When used in a clock or timer function call, this is interpreted as
 *  the identifier of the CPU_time clock associated with the PROCESS
 *  making the function call.
 */
#define CLOCK_PROCESS_CPUTIME (clockid_t)0 
 
#endif

#if defined(_POSIX_THREAD_CPUTIME)

/*
 *  When used in a clock or timer function call, this is interpreted as
 *  the identifier of the CPU_time clock associated with the THREAD
 *  making the function call.
 */
#define CLOCK_THREAD_CPUTIME (clockid_t)0 

#endif

#if defined(_POSIX_CPUTIME)

/*
 *  20.1.3 Accessing a Process CPU-time CLock, P1003.4b/D8, p. 55
 */

int clock_getcpuclockid(
  pid_t      pid,
  clockid_t *clock_id
);

#endif /* _POSIX_CPUTIME */

#if defined(_POSIX_CPUTIME) || defined(_POSIX_THREAD_CPUTIME)

/*
 *  20.1.5 CPU-time Clock Attribute Access, P1003.4b/D8, p. 58
 */

int clock_setenable_attr(
  clockid_t    clock_id,
  int          attr
);

int clock_getenable_attr(
  clockid_t    clock_id,
  int         *attr
);

#endif /* _POSIX_CPUTIME or _POSIX_THREAD_CPUTIME */

#endif
/* end of include file */
