/*  rtems/posix/intr.h
 *
 *  This include file contains all the private support information for
 *  POSIX Interrupt Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  intr.h,v 1.3 1995/12/19 20:00:49 joel Exp
 */
 
#ifndef __RTEMS_POSIX_KEY_h
#define __RTEMS_POSIX_KEY_h
 
#ifdef __cplusplus
extern "C" {
#endif

#include <rtems/score/isr.h>
#include <rtems/score/object.h>

/*
 *  Data Structure used to manage each POSIX Interrupt Vector
 */

typedef struct {
  int            number_installed;
  int            lock_count;
  int            deferred_count;
  Chain_Control  Handlers;
} POSIX_Interrupt_Control;
 
/*
 *  Data Structure used to manage a POSIX Interrupt Handler
 */
 
typedef struct {
  Objects_Control         Object;
  int                     is_active; 
  intr_t                  vector; 
  Thread_Control         *server;
  int                   (*handler)( void *area );
  volatile void           *user_data_area;
}  POSIX_Interrupt_Handler_control;

/*
 *  The following defines the information control block used to manage
 *  this class of objects.
 */
 
EXTERN Objects_Information _POSIX_Interrupt_Handlers_Information;

/*
 *  The following is an array which is used to manage the set of
 *  interrupt handlers installed on each vector.
 */

EXTERN POSIX_Interrupt_Control _POSIX_Interrupt_Information[ ISR_NUMBER_OF_VECTORS ];
 
/*
 *  _POSIX_Interrupt_Manager_initialization
 *
 *  DESCRIPTION:
 *
 *  This routine performs the initialization necessary for this manager.
 */
 
void _POSIX_Interrupt_Manager_initialization( 
  unsigned32  maximum_interrupt_handlers
);
 
/*
 *  _POSIX_Interrupt_Allocate
 *
 *  DESCRIPTION:
 *
 *  This function allocates a interrupt handler control block from
 *  the inactive chain of free interrupt handler control blocks.
 */
 
STATIC INLINE POSIX_Interrupt_Handler_control *
  _POSIX_Interrupt_Allocate( void );
 
/*
 *  _POSIX_Interrupt_Free
 *
 *  DESCRIPTION:
 *
 *  This routine frees a interrupt handler control block to the
 *  inactive chain of free interrupt handler control blocks.
 */
 
STATIC INLINE void _POSIX_Interrupt_Free (
  POSIX_Interrupt_Handler_control *the_intr
);
 
/*
 *  _POSIX_Interrupt_Get
 *
 *  DESCRIPTION:
 *
 *  This function maps interrupt handler IDs to interrupt handler control 
 *  blocks.  If ID corresponds to a local interrupt handler, then it returns
 *  the_intr control pointer which maps to ID and location
 *  is set to OBJECTS_LOCAL.  if the interrupt handler ID is global and
 *  resides on a remote node, then location is set to OBJECTS_REMOTE,
 *  and the_intr is undefined.  Otherwise, location is set
 *  to OBJECTS_ERROR and the_intr is undefined.
 */
 
STATIC INLINE POSIX_Interrupt_Control *_POSIX_Interrupt_Get (
  Objects_Id         id,
  Objects_Locations *location
);
 
/*
 *  _POSIX_Interrupt_Is_null
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the_intr is NULL and FALSE otherwise.
 */
 
STATIC INLINE boolean _POSIX_Interrupt_Is_null (
  POSIX_Interrupt_Handler_control *the_intr
);
 
/*
 *  _POSIX_Interrupt_Handler
 *
 *  DESCRIPTION:
 *
 *  This function XXX.
 */
 
void _POSIX_Interrupt_Handler(
  ISR_Vector_number   vector
);

#include <rtems/posix/intr.inl>

#ifdef __cplusplus
}
#endif
 
#endif
/*  end of include file */

