/*  sys/pthread.h
 *
 *  pthread.h,v 1.3 1995/12/19 20:01:53 joel Exp
 */

#ifndef __SYS_PTHREAD_h
#define __SYS_PTHREAD_h

#include <rtems/posix/features.h>

#if defined(_POSIX_THREADS)

#include <sys/types.h>
#include <sys/time.h>
#include <sys/sched.h>

#include <rtems/system.h>
#include <rtems/score/object.h>

/*
 *  2.5 Primitive System Data Types,  P1003.1c/D10, p. 19.
 */

typedef Objects_Id pthread_t;            /* identify a thread */

/*
 *  XXX
 *
 *  NOTE: P1003.4b/D8, p. 54 adds cputime_clock_allowed attribute.
 */

/* P1003.1c/D10, p. 118-119 */
#define PTHREAD_SCOPE_PROCESS 0
#define PTHREAD_SCOPE_SYSTEM  1

/* P1003.1c/D10, p. 141 */
#define PTHREAD_CREATE_DETACHED 0
#define PTHREAD__CREATE_JOINABLE  1

typedef struct {
  int is_initialized;
  void *stackaddr;
  int stacksize;
  int contentionscope;
  int inheritsched;
  int schedpolicy;
  struct sched_param schedparam;

#if defined(_POSIX_THREAD_CPUTIME)
  int  cputime_clock_allowed;  /* see time.h */
#endif
  int  detachstate;

} pthread_attr_t;

/*
 *  XXX
 */

#if defined(_POSIX_THREAD_PROCESS_SHARED)
/*
 *  NOTE: P1003.1c/D10, p. 81 defines following values for process_shared.
 */

#define PTHREAD_PROCESS_PRIVATE 0 /* visible within only the creating process */
#define PTHREAD_PROCESS_SHARED  1 /* visible too all processes with access to */
                                  /*   the memory where the resource is */
                                  /*   located */
#endif

/*
 * XXX
 */

typedef Objects_Id pthread_mutex_t;      /* identify a mutex */

typedef struct {
  int   is_initialized;
#if defined(_POSIX_THREAD_PROCESS_SHARED)
  int   process_shared;  /* allow mutex to be shared amongst processes */
#endif
#if defined(_POSIX_THREAD_PRIO_PROTECT)
  int   prio_ceiling;
  int   protocol;
#endif
  int   recursive;
} pthread_mutexattr_t;

/*
 *  XXX 
 */

typedef Objects_Id pthread_cond_t;       /* identify a condition variable */

typedef struct {
  int   is_initialized;
#if defined(_POSIX_THREAD_PROCESS_SHARED)
  int   process_shared;       /* allow this to be shared amongst processes */
#endif
} pthread_condattr_t;         /* a condition attribute object */

/*
 *  XXX 
 */

typedef Objects_Id pthread_key_t;        /* thread-specific data keys */

/*
 *  XXX 
 */

typedef struct {
  int   is_initialized;  /* is this structure initialized? */
  int   init_executed;   /* has the initialization routine been run? */
} pthread_once_t;       /* dynamic package initialization */

#endif /* defined(_POSIX_THREADS) */
#endif
/* end of include file */
