/* sys/siginfo.h
 *
 *  siginfo.h,v 1.3 1995/12/19 20:01:56 joel Exp
 */

#ifndef __POSIX_SYS_SIGNAL_INFORMATION_h
#define __POSIX_SYS_SIGNAL_INFORMATION_h

#if defined(_POSIX_THREADS)
#include <sys/pthread.h>
#endif

/*
 *  3.3  Signal Concepts, P1003.1b-1993, p. 60
 */
 
typedef int sigset_t;
 
#define SIG_DFL   -1 /* Request for default signal handling */
#define SIG_IGN   -1 /* Request that signal be ignored */
 
/*
 *  Required Signals.
 *
 *   The default action is in parentheses and is one of the
 *   following actions:
 *
 *     (1) abnormal termination of process
 *     (2) ignore the signal
 *     (3) stop the process
 *     (4) continue the process if it is currently stopped, otherwise
 *         ignore the signal
 */
 
#define SIGABRT  1  /* (1) abnormal terminal signal */
#define SIGALRM  2  /* (1) timeout signal, such as initiated by alarm() */
#define SIGFPE   3  /* (1) erroneous arithmetic operation */
#define SIGHUP   4  /* (1) hangup detected on controlling terminal */
#define SIGILL   5  /* (1) illegal instruction */
#define SIGINT   6  /* (1) interactive attention signal */
#define SIGKILL  7  /* (1) termination signal */
#define SIGPIPE  8  /* (1) write on pipe with no readers */
#define SIGQUIT  9  /* (1) interactive termination signal */
#define SIGSEGV 10  /* (1) invalid memory reference */
#define SIGTERM 11  /* (1) termination signal */
#define SIGUSR1 12  /* (1) reserved as application defined signal 1 */
#define SIGUSR2 13  /* (1) reserved as application defined signal 2 */
 
/*
 *  Job Control Signals
 */
 
#define SIGCHLD 14  /* (2) child process terminated or stopped */
#define SIGCONT 15  /* (4) continue if stopped */
#define SIGSTOP 16  /* (3) step signal (cannot be caught or ignored) */
#define SIGTSTP 17  /* (3) interactive stop signal */
#define SIGTTIN 18  /* (3) read from control terminal attempted by */
                    /*     a member of a background process */
#define SIGTTOU 19  /* (3) write to control terminal attempted by */
                    /*     a member of a background process */

/*
 *  Memory Protection Signals
 */
 
#define SIGBUS  20  /* (1) access to an undefined portion of a memory object */
 
/*
 *  Real-Time Signals Range, P1003.1b-1993, p. 61
 *
 *  NOTE: This should be at least RTSIG_MAX signals.
 */
 
#define SIGRTMIN 0
#define SIGRTMAX 0
 
/* sigev_notify values
 *
 *  NOTE: P1003.1c/D10, p. 34 adds SIGEV_THREAD.
 */
 
#define SIGEV_NONE   0  /* No asynchronous notification shall be delivered */
                        /*   when the event of interest occurs. */
#define SIGEV_SIGNAL 0  /* A queued signal, with an application defined */
                        /*  value, shall be delivered when the event of */
                        /*  interest occurs. */
#define SIGEV_THREAD 0  /* A notification function shall be called to */
                        /*   perform notification. */
/*
 *  3.3.1.2 Signal Generation and Delivery, P1003.1b-1993, p. 63
 *
 *  NOTE: P1003.1c/D10, p. 34 adds sigev_notify_function and 
 *        sigev_notify_attributes to the sigevent structure. 
 */

union sigval {
  int    sival_int;    /* Integer signal value */
  void  *sival_ptr;    /* Pointer signal value */
};

struct sigevent {
  int              sigev_notify;               /* Notification type */
  int              sigev_signo;                /* Signal number */
  union sigval     sigev_value;                /* Signal value */

#if defined(_POSIX_THREADS)
  void           (*sigev_notify_function)( union sigval ); 
                                               /* Notification function */
  pthread_attr_t  *sigev_notify_attributes;    /* Notification Attributes */
#endif
};

/*
 *  3.3.1.3 Signal Actions, P1003.1b-1993, p. 64
 */
 
/* si_code values, p. 66 */
 
#define SI_USER    0    /* Sent by a user. kill(), abort(), etc */
#define SI_QUEUE   0    /* Sent by sigqueue() */
#define SI_TIMER   0    /* Sent by expiration of a timer_settime() timer */
#define SI_ASYNCIO 0    /* Indicates completion of asycnhronous IO */
#define SI_MESGQ   0    /* Indicates arrival of a message at an empty queue */
 
typedef struct {
  int          si_signo;    /* Signal number */
  int          si_code;     /* Cause of the signal */
  union sigval si_value;    /* Signal value */
} siginfo_t;
 
/*
 *  3.3.4 Examine and Change Signal Action, P1003.1b-1993, p. 70
 */
 
/* sa_flags values */
 
#define SA_NOCLDSTOP 0   /* Do not generate SIGCHLD when children stop */
#define SA_SIGINFO   0   /* Invoke the signal catching function with */
                         /*   three arguments instead of one. */
 
/*
 *  Data Structure Notes:
 *
 *  (1) Routines stored in sa_handler should take a single int as
 *      there argument although the POSIX standard does not require this.
 *  (2) The fields sa_handler and sa_sigaction may overlap, and a conforming
 *      application should not use both simultaneously.
 */
 
struct sigaction {
  void      (*sa_handler)();  /* SIG_DFL, SIG_IGN, or pointer to a function */
  sigset_t    sa_mask;        /* Additional set of signals to be blocked */
                              /*   during execution of signal-catching */
                              /*   function. */
  int         sa_flags;       /* Special flags to affect behavior of signal */
  void      (*sa_sigaction)( int, siginfo_t *, void *);
                              /* Pointer to a function */
};
 
#endif
/* end of include file */
