/* sys/types.h
 *
 *  types.h,v 1.2 1995/10/06 20:46:52 joel Exp
 */

#ifndef __POSIX_SYS_TYPES_h
#define __POSIX_SYS_TYPES_h

#include <rtems/posix/features.h>

/*
 *  2.5: Primitive System Data Types, P1003.1b-1993, p. 32.
 */

typedef int dev_t;             /* device numbers */
typedef int gid_t;             /* group IDs */
typedef int inode_t;           /* file serial numbers */

/* XXX historical ... */
typedef unsigned long ino_t;

typedef int mode_t;            /* some file attributes */
typedef int nlink_t;           /* link counts */
typedef int off_t;             /* file sizes */
typedef int pid_t;             /* process and process group IDs */

#ifndef __SIZE_T
#define __SIZE_T
typedef int size_t;            /* see C Standard XXX */
#endif

typedef int ssize_t;           /* count of bytes (memory space) or error */
typedef int uid_t;             /* user IDs */

/*
 *  4.1.1 Get Process and Parent Process IDs, P1003.1b-1993, p. 83
 */

pid_t getpid( void );

pid_t getppid( void );

/*
 *  4.2.1 Get Real User, Effective User, Ral Group, and Effective Group IDs, 
 *        P1003.1b-1993, p. 84
 */

uid_t getuid( void );

uid_t geteuid( void );

gid_t getgid( void );

gid_t getegid( void );

/*
 *  4.2.2 Set User and Group IDs, P1003.1b-1993, p. 84
 */

int setuid(
  uid_t  uid
);

int setgid(
  gid_t  gid
);

/*
 *  4.2.3 Get Supplementary IDs, P1003.1b-1993, p. 86
 */

int getgroups(
  int    gidsetsize,
  gid_t  grouplist[]
);

/*
 *  4.2.4 Get User Name, P1003.1b-1993, p. 87
 *
 *  NOTE:  P1003.1c/D10, p. 49 adds getlogin_r().
 */

char *getlogin( void );

#if defined(_POSIX_THREAD_SAFE_FUNCTIONS)
char *getlogin_r( void );
#endif

/*
 *  4.3.1 Get Process Group IDs, P1003.1b-1993, p. 89
 */

pid_t getpgrp( void );

/*
 *  4.3.2 Create Session and Set Process Group ID, P1003.1b-1993, p. 88
 */

pid_t setsid( void );

/*
 *  4.3.3 Set Process Group ID for Job Control, P1003.1b-1993, p. 89
 */

int setpgid(
  pid_t  pid,
  pid_t  pgid
);

/*
 *  14.1.3 Clock Type Definition, P1003.1b-1993, p. 262
 */

typedef int clockid_t;
typedef int timer_t;

#endif
/* end of include file */
