/*  inthrd.h
 *
 *  This include file contains all the constants and structures associated
 *  with the threads which are internal to the executive. This handler 
 *  provides the internal tasks System_initialization_thread and the 
 *  Idle_thread which are used by the executive for internal control.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  intthrd.h,v 1.13 1995/12/19 20:02:12 joel Exp
 */

#ifndef __RTEMS_INTERNAL_THREADS_h
#define __RTEMS_INTERNAL_THREADS_h

#ifdef __cplusplus
extern "C" {
#endif

#include <rtems/score/stack.h>
#include <rtems/score/thread.h>
#include <rtems/score/wkspace.h>

/*
 *  The following constants define the stack size requirements for
 *  the system threads.
 */

#define INTERNAL_THREADS_SYSTEM_INITIALIZATION_THREAD_STACK_SIZE \
  ( STACK_MINIMUM_SIZE + \
    CPU_SYSTEM_INITIALIZATION_THREAD_EXTRA_STACK + \
    _CPU_Table.extra_system_initialization_stack \
  )

#define INTERNAL_THREADS_IDLE_THREAD_STACK_SIZE  STACK_MINIMUM_SIZE

/*
 *  The following defines the information control block used to
 *  manage this class of objects.
 */
 
EXTERN Objects_Information _Internal_threads_Information;

/*
 *  The following define the thread control pointers used to access
 *  and manipulate the system threads.
 */

EXTERN Thread_Control *_Internal_threads_System_initialization_thread;
EXTERN Thread_Control *_Internal_threads_Idle_thread;

/*
 *  _Internal_thread_Initialization
 *
 *  DESCRIPTION:
 *
 *  This routine initializes the internal thread handler.
 *
 */

void _Internal_threads_Initialization( void );

/*
 *  _Internal_thread_Start
 *
 *  DESCRIPTION:
 *
 *  This routine initializes the system threads.
 *
 */
 
void _Internal_threads_Start( void );

/*
 *  _Internal_threads_Initialize_user_tasks
 *
 *  DESCRIPTION:
 *
 *  This routine creates and starts all of the application tasks
 *  specified in the Initialization Tasks Table.
 */

void _Internal_threads_Initialize_user_tasks( void );

/*
 *  _Internal_threads_Allocate
 *
 *  DESCRIPTION:
 *
 *  This routine allocates an internal thread.
 */
 
STATIC INLINE Thread_Control *_Internal_threads_Allocate( void );

/*
 *  _Internal_threads_Free
 *
 *  DESCRIPTION:
 *
 *  This routine frees an internal thread.
 */
 
STATIC INLINE void _Internal_threads_Free (
  Thread_Control *the_task
);


/*
 *  _Internal_threads_System_initialization_thread_body
 *
 *  DESCRIPTION:
 *
 *  This routine is the body of the system initialization thread.
 */

Thread _Internal_threads_System_initialization_thread_body(
  unsigned32 ignored
);

/*
 *  _Internal_threads_Idle_thread_body
 *
 *  DESCRIPTION:
 *
 *  This routine is the body of the system idle thread.
 */

#if (CPU_PROVIDES_IDLE_THREAD_BODY == FALSE)
Thread _Internal_threads_Idle_thread_body(
  unsigned32 ignored
);
#endif

#include <rtems/intthrd.inl>
#include <rtems/inthrdmp.h>

#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
