/*  io.inl
 *
 *  This file contains static inline implementations for all of the inlined
 *  routines in the input/output manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  io.inl,v 1.6 1995/12/19 20:02:24 joel Exp
 */

#ifndef __IO_inl
#define __IO_inl

#include <rtems/score/wkspace.h>

/*PAGE
 *
 *  _IO_Manager_initialization
 *
 */

STATIC INLINE void _IO_Manager_initialization(
  rtems_driver_address_table *driver_table,
  unsigned32                  number_of_drivers,
  unsigned32                  number_of_devices
)
{
  void                *tmp;
  unsigned32           index;
  rtems_driver_name_t *np;

  _IO_Driver_address_table = driver_table;
  _IO_Number_of_drivers    = number_of_drivers;
  _IO_Number_of_devices    = number_of_devices;

  tmp = _Workspace_Allocate_or_fatal_error( 
    sizeof( rtems_driver_name_t ) * ( number_of_devices + 1 )
  );

  _IO_Driver_name_table = (rtems_driver_name_t *) tmp;

  for( index=0, np = _IO_Driver_name_table ;
       index < _IO_Number_of_devices ;
       index++, np++ ) {
    np->device_name = 0;
    np->device_name_length = 0;
    np->major = 0;
    np->minor = 0;
  }
}

#endif
/* end of include file */
