/*
 *  Multiprocessing Support for the Internal Threads
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  inthrdmp.c,v 1.7 1995/12/19 20:02:46 joel Exp
 */

#include <rtems/system.h>
#include <rtems/score/interr.h>
#include <rtems/intthrd.h>
#include <rtems/score/mpci.h>
#include <rtems/score/mppkt.h>
#include <rtems/score/object.h>
#include <rtems/score/states.h>
#include <rtems/score/thread.h>

/*PAGE
 *
 *  _Internal_threads_MP_Send_process_packet
 *
 */

void _Internal_threads_MP_Send_process_packet (
  Internal_threads_MP_Remote_operations operation
)
{
  Internal_threads_MP_Packet *the_packet;

  switch ( operation ) {

    case INTERNAL_THREADS_MP_SYSTEM_VERIFY:

      the_packet                    = _Internal_threads_MP_Get_packet();
      the_packet->Prefix.the_class  = MP_PACKET_INTERNAL_THREADS;
      the_packet->Prefix.length     = sizeof ( Internal_threads_MP_Packet );
      the_packet->Prefix.to_convert = sizeof ( Internal_threads_MP_Packet );
      the_packet->operation         = operation;

      the_packet->maximum_nodes = _Objects_Maximum_nodes;

      the_packet->maximum_global_objects = _Objects_MP_Maximum_global_objects;

      _MPCI_Send_process_packet( MPCI_ALL_NODES, &the_packet->Prefix );
      break;
  }
}

/*PAGE
 *
 *  _Internal_threads_MP_Send_request_packet
 *
 *  This subprogram is not needed since there are no request
 *  packets to be sent by this manager.
 *
 */

/*PAGE
 *
 *  _Internal_threads_MP_Send_response_packet
 *
 *  This subprogram is not needed since there are no response
 *  packets to be sent by this manager.
 *
 */

/*PAGE
 *
 *
 *  _Internal_threads_MP_Process_packet
 *
 */

void _Internal_threads_MP_Process_packet (
  MP_packet_Prefix  *the_packet_prefix
)
{
  Internal_threads_MP_Packet *the_packet;
  unsigned32                  maximum_nodes;
  unsigned32                  maximum_global_objects;

  the_packet = (Internal_threads_MP_Packet *) the_packet_prefix;

  switch ( the_packet->operation ) {

    case INTERNAL_THREADS_MP_SYSTEM_VERIFY:

      maximum_nodes          = the_packet->maximum_nodes;
      maximum_global_objects = the_packet->maximum_global_objects;
      if ( maximum_nodes != _Objects_Maximum_nodes ||
           maximum_global_objects != _Objects_MP_Maximum_global_objects ) {

        _MPCI_Return_packet( the_packet_prefix );

        _Internal_error_Occurred(
          INTERNAL_ERROR_CORE,
          TRUE,
          INTERNAL_ERROR_INCONSISTENT_MP_INFORMATION
        );
      }

      _MPCI_Return_packet( the_packet_prefix );

      break;
  }
}

/*PAGE
 *
 *  _Internal_threads_MP_Send_object_was_deleted
 *
 *  This subprogram is not needed since there are no objects
 *  deleted by this manager.
 *
 */

/*PAGE
 *
 *  _Internal_threads_MP_Send_extract_proxy
 *
 *  This subprogram is not needed since there are no objects
 *  deleted by this manager.
 *
 */

/*PAGE
 *
 *  _Internal_threads_MP_Get_packet
 *
 */

Internal_threads_MP_Packet *_Internal_threads_MP_Get_packet ( void )
{
  return ( (Internal_threads_MP_Packet *) _MPCI_Get_packet() );
}

/* end of file */
