/*
 *  RTEMS kill Implementation
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  __kill.c,v 1.3 1995/12/19 20:12:37 joel Exp
 */

#include <rtems.h>

#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#ifdef RTEMS_NEWLIB
#include <reent.h>
#endif
#include <unistd.h>

int
__kill(
  pid_t pid,
  int sig
)
{
  rtems_status_code status;
  rtems_id          id;
  rtems_unsigned32  ignored;

  id = (rtems_id) pid;

  if ( sig == 0 )
    status = rtems_task_get_note( id, RTEMS_NOTEPAD_0, &ignored );
  else if ( sig == SIGKILL )
    status = rtems_task_delete( id );
  else
    status = rtems_signal_send( id, 1 << (sig - 1) );

  if ( status == RTEMS_INVALID_ID ) {
    errno = ESRCH;
    return -1;
  } else if ( status != RTEMS_SUCCESSFUL ) {
    errno = EINVAL;
    return -1;
  }

  return 0;
}
